--############################################
-- Namespace
--############################################
local _, addon = ...;
addon.version = "0.1";
addon.DataToSave = {};
addon.DataToSave.charactersDatabase = {};
addon.DataToSave.options = {};
addon.CurrentCharacterInfo = {};

--Function that gets the time where the server resets
function addon:WeekResetTime() 
    --Get the second left for the reset of today
    local timeLeftForQuestReset = time() + GetQuestResetTime();
    local weeklyResetDays = {["us"] = 2, ["eu"] = 3, ["cn"] = 4, ["kr"] = 4, ["tw"] = 4};
    --Check if already on the reset day
    if(tonumber(date("%w", timeLeftForQuestReset)) ~= weeklyResetDays[GetCVar("portal"):lower()]) then
        --Not on reset day )= (You need to wait more to raid again T.T)
        --Time travel incoming :D
        repeat
            -- Add a day to the current time until we are at the reset day
            timeLeftForQuestReset = timeLeftForQuestReset + (24 * 60 * 60);
        until(tonumber(date("%w", timeLeftForQuestReset)) == weeklyResetDays[GetCVar("portal"):lower()]);
        -- Yep we are now on the time where the server resets are executed! (timeLeftForQuestReset is now on the exact date and houer)
    end
    return timeLeftForQuestReset;
end

--##########################################################################################################################
--                                  Helper Functions
--##########################################################################################################################

function addon:tableContains(t, value)
    for i,v in ipairs(t) do
        if(type(v) == "table") then
            tableContains(v, value);
        else
            return v == value;
        end
    end
end

function addon:Debug(...)
    addon:Print(string.join(" ", "|cFFFF0000(DEBUG)|r", tostringall(... or "nil")));
end

function addon:Print(...)
    msg = string.join(" ","|cFF029CFC[ClassHallsAccountReporter]|r", tostringall(... or "nil"));
    DEFAULT_CHAT_FRAME:AddMessage(msg);
end

-- Function to add the src table info to the dst talbe info, if dest already has this info overwrite it
function addon:addMissingFields(base, t)
    if(type(t) ~= "table") then
        return base;
    end

    if(type(base) ~= "table") then
        return t;
    end

    for k,v in pairs(base) do
        if(t[k] == nil and k ~= nil) then
            t[k] = v;
        end
    end
    return t;
end

function addon:SortcharacterList(t)
    local keys = {};
    for n,k in pairs(t) do
        table.insert(keys, n);
    end
    table.sort(keys, function(a,b)
        local name1,server1 = strsplit("-", a, 2);
        local name2,server2 = strsplit("-", b, 2);
        if(server1 == server2) then
            --Short by name
            local ComputF8 = strcmputf8i(name1, name2);
            return ComputF8 < 0;
        else
            --Different server so yhea put all the same server together (?)
            return true;
        end
    end);

    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

function addon:SortFollowers(t)
    local statusTextsPriotities = {
        [GARRISON_FOLLOWER_IN_PARTY] = 1,
        [GARRISON_FOLLOWER_WORKING] = 2,
        [GARRISON_FOLLOWER_ON_MISSION] = 3,
        [GARRISON_FOLLOWER_COMBAT_ALLY] = 4,
        [GARRISON_FOLLOWER_EXHAUSTED] = 5,
        [GARRISON_FOLLOWER_INACTIVE] = 6
    };

    local keys = {};
    for n,k in pairs(t) do
        table.insert(keys, n);
    end
    table.sort(keys, function(a,b)
        if(t[a]["status"] and not t[b]["status"]) then
            return false;
        elseif(not t[a]["status"] and t[b]["status"]) then
            return true;
        end
        if(t[a]["status"] ~= t[b]["status"]) then
            return statusTextsPriotities[t[a]["status"]] < statusTextsPriotities[t[b]["status"]];
        end
        if(t[a]["level"] ~= t[b]["level"]) then
            return t[a]["level"] > t[b]["level"];
        elseif(t[a]["ilvl"] ~= t[b]["ilvl"]) then
            return t[a]["ilvl"] > t[b]["ilvl"];
        end
        if(t[a]["quality"] ~= t[b]["quality"]) then
            return t[a]["quality"] > t[b]["quality"];
        end
        if(t[a]["xp"] > 0 and t[b]["xp"] > 0) then
            return t[a]["xp"] > t[b]["xp"];
        end
        return t[a]["garrFollowerID"] > t[b]["garrFollowerID"];
    end);

    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

function addon:SortMissions(t)
    local keys = {};
    for n,k in pairs(t) do
        table.insert(keys, n);
    end
    table.sort(keys, function(a,b)
        return t[a].missionEndTime < t[b].missionEndTime;
    end);

    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

-- Returns a string of the time formated
function addon:convertSecondToTimeStr(s)
    return SecondsToTime(s);
end

-- Fixes the info about shipments and save it again
function addon:fixShipmentsInfo(shipment)
    local currentFinishedShipmentsTime = shipment.creationTime + (shipment.duration*shipment.shipmentsReady);
           
    while(currentFinishedShipmentsTime + shipment.duration < time() and shipment.shipmentsReady < shipment.shipmentsTotal)  do
        currentFinishedShipmentsTime = currentFinishedShipmentsTime +  shipment.duration;
        shipment.shipmentsReady = shipment.shipmentsReady +1;
    end
end

function addon:prepareCharacterDataBase()
    local name = select(1, UnitName("player")).."-"..GetRealmName();
    -- Check if the current character already exits in the table if not create a table for him
    if(addon.DataToSave.charactersDatabase.characters[name] == nil or type(addon.DataToSave.charactersDatabase.characters[name]) ~= "table") then
       addon.DataToSave.charactersDatabase.characters[name] = {};

        -- Fix the current table database to add missing fields to have all the required fields 
        addon.DataToSave.charactersDatabase.characters[name] = addon:addMissingFields(
        {
            pclassName = select(2, UnitClass("player")),
            pclassID = select(3, UnitClass("player")),
            plevel =  UnitLevel("player"),
            pilvl = select(2, GetAverageItemLevel()),
            lastTimeUpdated = time(),
            currency = {},
            troops = {},
            followers = {},
            activeMissions = {},
            availableMissions = {},
            troopsInTraining = {},
            shipments = {}
        } , addon.DataToSave.charactersDatabase.characters[name]);
    end
    -- Set the info of the current character
    addon.CurrentCharacterInfo = addon.DataToSave.charactersDatabase.characters[name];
end

--##########################################################################################################################
--                                  Functions to get Players Info
--##########################################################################################################################


-- Function to get the order resources and broken seals
function addon:storeCurrencyInfo()
    if (not C_Garrison.HasGarrison( LE_GARRISON_TYPE_7_0 )) then
        return;
    end
    -- Prepare the character if necesary
    addon:prepareCharacterDataBase();
    -- Get resources and Seals of Broken Fate amount for the character
    local _, amountResources = GetCurrencyInfo(1220);
    local _, amountSeal =  GetCurrencyInfo(1273);
    local _, amountnethershards = GetCurrencyInfo(1226);
    local _, amountcuriouscoin = GetCurrencyInfo(1275);
    local _, amountveiledargunite = GetCurrencyInfo(1508);
    addon.CurrentCharacterInfo.currency =  {
        ["HallResources"] = amountResources,
        ["SealsOfBrokenFate"] = amountSeal,
        ["bloodOfSargeras"] = GetItemCount(124124, true),
        ["gold"] = (GetMoney()/100)/100,
        ["nethershards"] = amountnethershards,
        ["curiouscoin"] = amountcuriouscoin,
        ["veiledargunite"] = amountveiledargunite,
        ["SealsMissionsCompleted"] = {}
    };
    -- Save the quest completed by the character this week
    local BrokenFateQuests = { [43892] = {
                                            ["name"] = "Sealing Fate: Order Resources",
                                            ["costStr"] = "1000 Order Resources" 
                                       },
                               [43893] = {
                                            ["name"] = "Sealing Fate: Stashed Order Resources",
                                            ["costStr"] = "2000 Order Resources" 
                                       },
                               [43894] = {
                                            ["name"] = "Sealing Fate: Extraneous Order Resources",
                                            ["costStr"] = "4000 Order Resources" 
                                       },
                               [43895] = {
                                            ["name"] = "Sealing Fate: Gold",
                                            ["costStr"] = "1000 gold" 
                                       },
                               [43896] = {
                                            ["name"] = "Sealing Fate: Piles of Gold",
                                            ["costStr"] = "2000 gold" 
                                       },
                               [43897] = {
                                            ["name"] = "Sealing Fate: Immense Fortune of Gold",
                                            ["costStr"] = "4000 gold" 
                                       },
                               [47851] = {
                                            ["name"] = "Sealing Fate: Marks of Honor",
                                            ["costStr"] = "5 Marks of Honor" 
                                       },
                               [47864] = {
                                            ["name"] = "Sealing Fate: Additional Marks of Honor",
                                            ["costStr"] = "10 Marks of Honor" 
                                       },
                               [47865] = {
                                            ["name"] = "Sealing Fate: Piles of Marks of Honor",
                                            ["costStr"] = "20 Marks Of Honor" 
                                       }
                               };
    addon.CurrentCharacterInfo.currency.SealsMissionsCompleted = {};
    for qid,info in pairs(BrokenFateQuests) do
        if(IsQuestFlaggedCompleted(qid)) then
            addon.CurrentCharacterInfo.currency.SealsMissionsCompleted[qid] = info;
        end
    end
    addon.CurrentCharacterInfo.lastTimeUpdated = time();
end

-- Function to get the class troop info
function addon:storeClassTroopInfo()
    -- Get info from the followers in you class hall
    local temp =  C_Garrison.GetClassSpecCategoryInfo(LE_FOLLOWER_TYPE_GARRISON_7_0);
    if(temp ~= nil and type(temp) == "table") then
        addon.CurrentTroopsClassInfo = temp;
        addon:storeFollowersAndTroopsInfo();
    end
end

--Function to store troops in training
function addon:storeToopsInTrainingInfo()
    if (not C_Garrison.HasGarrison( LE_GARRISON_TYPE_7_0 )) then
        return;
    end
    -- Prepare the character if necesary
    addon:prepareCharacterDataBase();

    -- Get training troops
    wipe(addon.CurrentCharacterInfo.troopsInTraining);
    local TrainingTroops = C_Garrison.GetFollowerShipments(LE_GARRISON_TYPE_7_0);
    if(TrainingTroops ~= nil and type(TrainingTroops) == "table") then
        for i, troopTrainingID in ipairs(TrainingTroops) do
            local  name, texture, shipmentCapacity, shipmentsReady, shipmentsTotal, creationTime, duration, timeleftString, _, _, _, _, followerID = C_Garrison.GetLandingPageShipmentInfoByContainerID(troopTrainingID);
            if(name ~= nil) then
                addon.CurrentCharacterInfo.troopsInTraining[troopTrainingID] = {
                    ["name"] = name,
                    ["texture"] = texture,
                    ["shipmentCapacity"] = shipmentCapacity,
                    ["shipmentsReady"] = shipmentsReady,
                    ["shipmentsTotal"] = shipmentsTotal,
                    ["creationTime"] = creationTime,
                    ["duration"] = duration
                };
            end
        end
    end
    addon.CurrentCharacterInfo.lastTimeUpdated = time();
end

-- Function to get the followers and troops info
function addon:storeFollowersAndTroopsInfo()
    if (not C_Garrison.HasGarrison( LE_GARRISON_TYPE_7_0 )) then
        return;
    end
    -- Prepare the character if necesary
    addon:prepareCharacterDataBase();
    -- Set the minimum required info to the troops
    wipe(addon.CurrentCharacterInfo.troops);
    if(addon.CurrentTroopsClassInfo ~= nil and type(addon.CurrentTroopsClassInfo) == "table") then
        for i,info in pairs(addon.CurrentTroopsClassInfo) do
            if(addon.CurrentCharacterInfo.troops[info.name] == nil) then
                addon.CurrentCharacterInfo.troops[info.name] = {
                        ["quality"] = nil,
                        ["maxDurability"] = nil,
                        ["maxCount"] = nil,
                        ["description"] = nil,
                        ["currentCount"] = nil,
                        ["classSpec"] = nil,
                        ["iconID"] = nil,
                        ["troopsArray"] = {}
                };
            end
            addon.CurrentCharacterInfo.troops[info.name].maxCount = info.limit;
            addon.CurrentCharacterInfo.troops[info.name].currentCount = info.count;
            addon.CurrentCharacterInfo.troops[info.name].description = info.description;
            addon.CurrentCharacterInfo.troops[info.name].iconID = info.icon;
            addon.CurrentCharacterInfo.troops[info.name].classSpec = info.classSpec;
        end
    end
    -- Get Followrs and troops, in case of the troops get the durability and if they are in a missions
    wipe(addon.CurrentCharacterInfo.followers);
    local followersAllData = C_Garrison.GetFollowers(LE_FOLLOWER_TYPE_GARRISON_7_0);
    if(followersAllData ~= nil and type(followersAllData) == "table") then
        for i, follower in ipairs(followersAllData) do
            if(follower.isCollected) then
                if(not follower.isTroop) then
                    addon.CurrentCharacterInfo.followers[follower.followerID] = {
                        ["garrFollowerID"] = follower.garrFollowerID,
                        ["name"] = follower.name,
                        ["level"] = follower.level,
                        ["xp"] = follower.xp,
                        ["levelXP"] = follower.levelXP,
                        ["quality"] = follower.quality,
                        ["ilvl"] = follower.iLevel,
                        ["isMaxLevel"] = follower.isMaxLevel,
                        ["status"] = follower.status,
                        ["classAtlas"] = follower.classAtlas,
                        ["iconID"] = follower.portraitIconID
                    };
                else
                    if(addon.CurrentCharacterInfo.troops[follower.name] ~= nil and type(addon.CurrentCharacterInfo.troops[follower.name]) == "table") then
                        if(addon.CurrentCharacterInfo.troops[follower.name].quality == nil) then
                            addon.CurrentCharacterInfo.troops[follower.name].quality = follower.quality;
                        end
                        if(addon.CurrentCharacterInfo.troops[follower.name].maxDurability == nil) then
                            addon.CurrentCharacterInfo.troops[follower.name].maxDurability = follower.maxDurability;
                        end

                        addon.CurrentCharacterInfo.troops[follower.name].troopsArray[follower.followerID] ={
                            ["garrFollowerID"] = follower.garrFollowerID,
                            ["currentDurability"] = follower.durability,
                            ["status"] = follower.status,
                            ["classAtlas"] =  follower.classAtlas
                        };
                    end
                end
            end
        end
    end
    addon:storeToopsInTrainingInfo();
    addon.CurrentCharacterInfo.lastTimeUpdated = time();
end

function addon:storeShipmentsInfo()
    if (not C_Garrison.HasGarrison( LE_GARRISON_TYPE_7_0 )) then
        return;
    end
    -- Prepare the character if necesary
    addon:prepareCharacterDataBase();
    --Get Acive missions (Name, Reward, Time left to complete, ILVL, Followers/troops in the mission)
    wipe(addon.CurrentCharacterInfo.activeMissions);
    local missionsInProgress = C_Garrison.GetLandingPageItems(LE_GARRISON_TYPE_7_0);
    local tempMissionInProgress = {};
    for i, v in ipairs(missionsInProgress) do
        v["timeInfoCollected"] = time();
        tempMissionInProgress[v.name] = v;
    end
    addon.CurrentCharacterInfo.activeMissions = tempMissionInProgress;

    -- Get Available missions (Name, Reward , Time to complete, Time to expire, ILVL)
    wipe(addon.CurrentCharacterInfo.availableMissions);
    local missionsAvailableToStart = C_Garrison.GetAvailableMissions(4);
    local tempMission = {};
    for i, v in ipairs(missionsAvailableToStart) do
        v["timeInfoCollected"] = time();
        tempMission[v.name] = v;
    end
    addon.CurrentCharacterInfo.availableMissions = tempMission;

    addon:storeToopsInTrainingInfo();

    -- Get shipments
   wipe(addon.CurrentCharacterInfo.shipments);
    local allShipments = C_Garrison.GetLooseShipments(LE_GARRISON_TYPE_7_0);
    if(allShipments ~= nil and type(allShipments) == "table") then
        for i, shipmentID in ipairs(allShipments) do
            local name, texture, shipmentCapacity, shipmentsReady, shipmentsTotal, creationTime, duration = C_Garrison.GetLandingPageShipmentInfoByContainerID(shipmentID);
            if(name ~= nil) then
                addon.CurrentCharacterInfo.shipments[shipmentID] = {
                    ["name"] = name,
                    ["texture"] = texture,
                    ["shipmentCapacity"] = shipmentCapacity,
                    ["shipmentsReady"] = shipmentsReady,
                    ["shipmentsTotal"] = shipmentsTotal,
                    ["creationTime"] = creationTime,
                    ["duration"] = duration
                };
                if(shipmentID == 122) then
                    addon.CurrentCharacterInfo.shipments[122].QuestCompleated = true; 
                end
            end
        end
    end
    if(addon.CurrentCharacterInfo.shipments[122] == nil) then
        addon.CurrentCharacterInfo.shipments[122] = {
            ["QuestCompleated"] = IsQuestFlaggedCompleted(40991);
        };
    end
    if(addon.CurrentCharacterInfo.shipments[122].QuestCompleated == nil) then
        addon.CurrentCharacterInfo.shipments[122].QuestCompleated  = IsQuestFlaggedCompleted(40991);
    end
    addon.CurrentCharacterInfo.lastTimeUpdated = time();
end


-- Function to store the characters hall class info
function addon:allInfoUpdater()
    if (C_Garrison.HasGarrison( LE_GARRISON_TYPE_7_0 )) then
        -- Prepare the character if necesary
        addon:prepareCharacterDataBase();
        C_Garrison.RequestLandingPageShipmentInfo();
        addon.storeShipmentsInfo();
        addon.storeFollowersAndTroopsInfo();
        addon.storeCurrencyInfo();
    end
end


function addon:resetAllCharactersWeeklyData()
    addon.DataToSave.charactersDatabase.nextReset = addon.WeekResetTime();
    for i,v in ipairs(addon.DataToSave.charactersDatabase.characters) do
        v["SealsMissionsCompleted"] = {};
        v["lastTimeUpdated"] = time();
    end
    
end